' Include file for users of PbSlice200.dll

' ===============================================
' Enums
Enum TokenTypes
	eUnknown     ' Unknown token (should never happen)
	eAsm         ' Token contains assembler statement
	eKeyword     ' Token is a built in
	eContinue    ' Token is the _
	eConstant    ' Token is a contant
	eIdentifier  ' Token is an identifier
	eNumber      ' Token is a number
	eOperator    ' Token is an operator
	eRemark      ' Token is a remark
	eString      ' Token is a string
End Enum

Enum KeywordCase
	Original ' Builtin keywords remain as typed
	Upper    ' Builtin keywords are upper cased
	Lower    ' Builtin keywords are lower cased
	Mixed    ' Builtin keywords are mixed cased
End Enum

Enum DeclarationTypes
	eUnknown
	eAsmData        ' Declaration is an AsmData
	eClass          ' Declaration is a class
	eClassMethod    ' Declaration is a class method
	eConstant       ' Declaration is a constant
	eEnum           ' Declaration is am Enum constant
	eField          ' Declaration is a Type or Union field
	eFastProc       ' Declaration is a fast proc
	eFunction       ' Declaration is a function
	eInterface      ' Declaration is an interface
	eLabel          ' Declaration is a label
	eMacroFunction  ' Declaration is a macro function
	eMacro          ' Declaration is a macro
	eMacroTemp      ' Declaration is a Macro Temp
	eMethod         ' Declaration is a method
	eProperty       ' Declaration is a property get
	eReference      ' Declaration is a reference
	eSub            ' Declaration is a sub
	eString         ' Declaration is a String value
	eType           ' Declaration is a type
	eUnion          ' Declaration is a union
	eVariable       ' Declaration is a variable
	eLastType       ' Last entry, all entries should be before this
End Enum

' ===============================================
' Types
Type TDecl
	tFileNumber As Integer:   ' File number
	tTokenNumber As Integer:  ' Token number
	tLineNumber As Long:      ' Line number in file
	tNameIndex As Long:       ' Declaration name index
	tDeclType As Long:        ' Type of declaration (Function, Sub, etc) see Enum DeclarationType
	tParent As Long:          ' Class index for interfaces, interface index for props/meths
End Type

' ===============================================
' Routines
Declare Sub      pbsFileActivate Lib "PbSlice200.dll" Alias "pbsFileActivate"( ByVal fileNumber As Long )
Declare Sub      pbsFileAddText Lib "PbSlice200.dll" Alias "pbsFileAddText"( insertLines As String )
Declare Function pbsFileBuildDLLInclude Lib "PbSlice200.dll" Alias "pbsFileBuildDLLInclude"()
Declare Function pbsFileBuildSLLInclude Lib "PbSlice200.dll" Alias "pbsFileBuildSLLInclude"()
Declare Function pbsFileByteSize Lib "PbSlice200.dll" Alias "pbsFileByteSize"()
Declare Sub      pbsFileCasedText Lib "PbSlice200.dll" Alias "pbsFileCasedText"( ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long )
Declare Sub      pbsFileClose Lib "PbSlice200.dll" Alias "pbsFileClose"( ByVal fileNumber As Long )
Declare Function pbsFileDeclareGetAllInterfaces Lib "PbSlice200.dll" Alias "pbsFileDeclareGetAllInterfaces"()
Declare Function pbsFileDeclareGetNames Lib "PbSlice200.dll" Alias "pbsFileDeclareGetNames"( ByVal tp As Long )
Declare Sub      pbsFileDeclareScan Lib "PbSlice200.dll" Alias "pbsFileDeclareScan"( Optional ByVal pClear As Long )
Declare Sub      pbsFileDeleteChars Lib "PbSlice200.dll" Alias "pbsFileDeleteChars"( ByVal lineNumber As Long, ByVal index As Long, ByVal cnt As Long )
Declare Sub      pbsFileDeleteText Lib "PbSlice200.dll" Alias "pbsFileDeleteText"( ByVal startAt As Long, ByVal cnt As Long )
Declare Sub      pbsFileExpandTabs Lib "PbSlice200.dll" Alias "pbsFileExpandTabs"( Optional ByVal tabSize As Long )
Declare Function pbsFileFindExports Lib "PbSlice200.dll" Alias "pbsFileFindExports"()
Declare Function pbsFileGetFilename Lib "PbSlice200.dll" Alias "pbsFileGetFilename"()
Declare Function pbsFileGetFilenumber Lib "PbSlice200.dll" Alias "pbsFileGetFilenumber"()
Declare Function pbsFileGetIsExternal Lib "PbSlice200.dll" Alias "pbsFileGetIsExternal"()
Declare Function pbsFileGetLine Lib "PbSlice200.dll" Alias "pbsFileGetLine"( ByVal lineNumber As Long )
Declare Function pbsFileGetLineCount Lib "PbSlice200.dll" Alias "pbsFileGetLineCount"()
Declare Function pbsFileGetLines Lib "PbSlice200.dll" Alias "pbsFileGetLines"( ByVal lineNumber As Long, ByVal cnt As Long )
Declare Function pbsFileGetText Lib "PbSlice200.dll" Alias "pbsFileGetText"()
Declare Sub      pbsFileGetTextArray Lib "PbSlice200.dll" Alias "pbsFileGetTextArray"( ByVal lineNumber As Long, ByVal cnt As Long, values() As String )
Declare Sub      pbsFileIndent Lib "PbSlice200.dll" Alias "pbsFileIndent"()
Declare Function pbsFileIndexFilename Lib "PbSlice200.dll" Alias "pbsFileIndexFilename"( value As String )
Declare Sub      pbsFileInsertChars Lib "PbSlice200.dll" Alias "pbsFileInsertChars"( ByVal lineNumber As Long, ByVal index As Long, insertString As String )
Declare Sub      pbsFileInsertClass Lib "PbSlice200.dll" Alias "pbsFileInsertClass"( ByVal lineNumber As Long, className As String, Optional ByVal addGUID As Long, Optional ByVal isCommon As Long )
Declare Sub      pbsFileInsertInterface Lib "PbSlice200.dll" Alias "pbsFileInsertInterface"( ByVal lineNumber As Long, interfaceName As String, inheritFrom As String, Optional ByVal addGUID As Long )
Declare Sub      pbsFileInsertText Lib "PbSlice200.dll" Alias "pbsFileInsertText"( ByVal insertAt As Long, insertLines As String )
Declare Sub      pbsFileLoad Lib "PbSlice200.dll" Alias "pbsFileLoad"( filename As String )
Declare Sub      pbsFileManageBlankLines Lib "PbSlice200.dll" Alias "pbsFileManageBlankLines"()
Declare Sub      pbsFileNew Lib "PbSlice200.dll" Alias "pbsFileNew"( filename As String, Optional overwrite As Long )
Declare Sub      pbsFileReload Lib "PbSlice200.dll" Alias "pbsFileReload"()
Declare Sub      pbsFileRemoveContinuation Lib "PbSlice200.dll" Alias "pbsFileRemoveContinuation"()
Declare Sub      pbsFileReplaceChars Lib "PbSlice200.dll" Alias "pbsFileReplaceChars"( ByVal lineNumber As Long, ByVal index As Long, ByVal cnt As Long, insertString As String )
Declare Sub      pbsFileReplaceText Lib "PbSlice200.dll" Alias "pbsFileReplaceText"( ByVal startAt As Long, ByVal cnt As Long, insertLines As String )
Declare Sub      pbsFileReplaceToken Lib "PbSlice200.dll" Alias "pbsFileReplaceToken"( originalToken As String, newToken As String )
Declare Sub      pbsFileSave Lib "PbSlice200.dll" Alias "pbsFileSave"( Optional newFilename As String )
Declare Sub      pbsFileSetFilename Lib "PbSlice200.dll" Alias "pbsFileSetFilename"( value As String )
Declare Sub      pbsFileSetIsExternal Lib "PbSlice200.dll" Alias "pbsFileSetIsExternal"( ByVal value As Long )
Declare Sub      pbsFileSetLine Lib "PbSlice200.dll" Alias "pbsFileSetLine"( ByVal lineNumber As Long, value As String )
Declare Sub      pbsFileSetLineCount Lib "PbSlice200.dll" Alias "pbsFileSetLineCount"( ByVal value As Long )
Declare Sub      pbsFileSetText Lib "PbSlice200.dll" Alias "pbsFileSetText"( value As String )
Declare Function pbsFileShortname Lib "PbSlice200.dll" Alias "pbsFileShortname"( filename As String )
Declare Function pbsFileToBBCode Lib "PbSlice200.dll" Alias "pbsFileToBBCode"( ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long, ByVal includeMonoFormat As Long, ByVal includeColorFormat As Long )
Declare Function pbsFileToHtml Lib "PbSlice200.dll" Alias "pbsFileToHtml"( ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long, ByVal includeMonoFormat As Long, ByVal includeColorFormat As Long )
Declare Sub      pbsFileTrimWhitespace Lib "PbSlice200.dll" Alias "pbsFileTrimWhitespace"()
Declare Function pbsFilenameIsExternal Lib "PbSlice200.dll" Alias "pbsFilenameIsExternal"( filename As String )
Declare Sub      pbsFilesystemEnsureFilePath Lib "PbSlice200.dll" Alias "pbsFilesystemEnsureFilePath"( filename As String )
Declare Sub      pbsFilesystemEnsurePath Lib "PbSlice200.dll" Alias "pbsFilesystemEnsurePath"( fullPath As String )
Declare Function pbsKeywordExists Lib "PbSlice200.dll" Alias "pbsKeywordExists"( value As String )
Declare Function pbsKeywordGetFileList Lib "PbSlice200.dll" Alias "pbsKeywordGetFileList"()
Declare Function pbsKeywordGetFilename Lib "PbSlice200.dll" Alias "pbsKeywordGetFilename"()
Declare Function pbsKeywordGetKeywords Lib "PbSlice200.dll" Alias "pbsKeywordGetKeywords"( keywordName As String )
Declare Sub      pbsKeywordLoad Lib "PbSlice200.dll" Alias "pbsKeywordLoad"( keywordName As String )
Declare Sub      pbsKeywordSaveKeywords Lib "PbSlice200.dll" Alias "pbsKeywordSaveKeywords"( keywordName As String, keywords As String )
Declare Function pbsKeywordStandardize Lib "PbSlice200.dll" Alias "pbsKeywordStandardize"( value As String )
Declare Sub      pbsProjectAddDebugAliases Lib "PbSlice200.dll" Alias "pbsProjectAddDebugAliases"()
Declare Function pbsProjectBuildDLLInclude Lib "PbSlice200.dll" Alias "pbsProjectBuildDLLInclude"()
Declare Function pbsProjectBuildSLLInclude Lib "PbSlice200.dll" Alias "pbsProjectBuildSLLInclude"()
Declare Function pbsProjectByteCount Lib "PbSlice200.dll" Alias "pbsProjectByteCount"()
Declare Sub      pbsProjectClose Lib "PbSlice200.dll" Alias "pbsProjectClose"()
Declare Sub      pbsProjectCloseFile Lib "PbSlice200.dll" Alias "pbsProjectCloseFile"( ByVal fileNumber As Long )
Declare Sub      pbsProjectDeclareClear Lib "PbSlice200.dll" Alias "pbsProjectDeclareClear"()
Declare Function pbsProjectDeclareCount Lib "PbSlice200.dll" Alias "pbsProjectDeclareCount"()
Declare Function pbsProjectDeclareFindName Lib "PbSlice200.dll" Alias "pbsProjectDeclareFindName"( ByVal start As Long, value As String, ByVal declType As Long )
Declare Function pbsProjectDeclareGetAllInterfaces Lib "PbSlice200.dll" Alias "pbsProjectDeclareGetAllInterfaces"()
Declare Function pbsProjectDeclareGetClassInterfaces Lib "PbSlice200.dll" Alias "pbsProjectDeclareGetClassInterfaces"( ByVal index As Long )
Declare Function pbsProjectDeclareGetClassMethods Lib "PbSlice200.dll" Alias "pbsProjectDeclareGetClassMethods"( ByVal index As Long )
Declare Function pbsProjectDeclareGetClassOutline Lib "PbSlice200.dll" Alias "pbsProjectDeclareGetClassOutline"( ByVal index As Long )
Declare Function pbsProjectDeclareGetDeclarationName Lib "PbSlice200.dll" Alias "pbsProjectDeclareGetDeclarationName"( ByVal index As Long )
Declare Function pbsProjectDeclareGetDefinition Lib "PbSlice200.dll" Alias "pbsProjectDeclareGetDefinition"( ByVal index As Long )
Declare Function pbsProjectDeclareGetInstanceVariables Lib "PbSlice200.dll" Alias "pbsProjectDeclareGetInstanceVariables"( ByVal index As Long )
Declare Sub      pbsProjectDeclareGetItem Lib "PbSlice200.dll" Alias "pbsProjectDeclareGetItem"( ByVal index As Long, value As TDecl )
Declare Function pbsProjectDeclareGetName Lib "PbSlice200.dll" Alias "pbsProjectDeclareGetName"( ByVal index As Long )
Declare Function pbsProjectDeclareGetNames Lib "PbSlice200.dll" Alias "pbsProjectDeclareGetNames"( ByVal declType As Long )
Declare Function pbsProjectDeclareGetSignature Lib "PbSlice200.dll" Alias "pbsProjectDeclareGetSignature"( ByVal index As Long )
Declare Sub      pbsProjectDeclareScan Lib "PbSlice200.dll" Alias "pbsProjectDeclareScan"()
Declare Sub      pbsProjectExpandTabs Lib "PbSlice200.dll" Alias "pbsProjectExpandTabs"( Optional ByVal tabSize As Long )
Declare Function pbsProjectFileCount Lib "PbSlice200.dll" Alias "pbsProjectFileCount"()
Declare Function pbsProjectFindExports Lib "PbSlice200.dll" Alias "pbsProjectFindExports"()
Declare Sub      pbsProjectGetBasePath Lib "PbSlice200.dll" Alias "pbsProjectGetBasePath"( projectPath As String )
Declare Function pbsProjectGetFileNumber Lib "PbSlice200.dll" Alias "pbsProjectGetFileNumber"( filename As String )
Declare Function pbsProjectGetFilename Lib "PbSlice200.dll" Alias "pbsProjectGetFilename"( ByVal index As Long )
Declare Sub      pbsProjectGetIncludePaths Lib "PbSlice200.dll" Alias "pbsProjectGetIncludePaths"( paths As String )
Declare Function pbsProjectGetInterfaceClasses Lib "PbSlice200.dll" Alias "pbsProjectGetInterfaceClasses"( interfaceName As String )
Declare Function pbsProjectGetInterfaceRoutines Lib "PbSlice200.dll" Alias "pbsProjectGetInterfaceRoutines"( ByVal index As Long )
Declare Function pbsProjectGetInterfaceStub Lib "PbSlice200.dll" Alias "pbsProjectGetInterfaceStub"( ByVal index As Long )
Declare Function pbsProjectGetTabsize Lib "PbSlice200.dll" Alias "pbsProjectGetTabsize"()
Declare Function pbsProjectHasFile Lib "PbSlice200.dll" Alias "pbsProjectHasFile"( filename As String )
Declare Sub      pbsProjectIndent Lib "PbSlice200.dll" Alias "pbsProjectIndent"()
Declare Function pbsProjectLineCount Lib "PbSlice200.dll" Alias "pbsProjectLineCount"()
Declare Sub      pbsProjectLoad Lib "PbSlice200.dll" Alias "pbsProjectLoad"( mainFile As String )
Declare Sub      pbsProjectManageBlankLines Lib "PbSlice200.dll" Alias "pbsProjectManageBlankLines"()
Declare Sub      pbsProjectNew Lib "PbSlice200.dll" Alias "pbsProjectNew"( mainFile As String )
Declare Sub      pbsProjectReload Lib "PbSlice200.dll" Alias "pbsProjectReload"()
Declare Sub      pbsProjectRemoveAliases Lib "PbSlice200.dll" Alias "pbsProjectRemoveAliases"()
Declare Sub      pbsProjectRemoveContinuation Lib "PbSlice200.dll" Alias "pbsProjectRemoveContinuation"()
Declare Sub      pbsProjectReplaceToken Lib "PbSlice200.dll" Alias "pbsProjectReplaceToken"( originalToken As String, newToken As String )
Declare Sub      pbsProjectSave Lib "PbSlice200.dll" Alias "pbsProjectSave"()
Declare Sub      pbsProjectSetBasePath Lib "PbSlice200.dll" Alias "pbsProjectSetBasePath"( projectPath As String )
Declare Sub      pbsProjectSetFilename Lib "PbSlice200.dll" Alias "pbsProjectSetFilename"( ByVal index As Long, filename As String )
Declare Sub      pbsProjectSetIncludePaths Lib "PbSlice200.dll" Alias "pbsProjectSetIncludePaths"( paths As String )
Declare Sub      pbsProjectSetTabsize Lib "PbSlice200.dll" Alias "pbsProjectSetTabsize"( ByVal tabSize As Long )
Declare Sub      pbsProjectStandardizeExportAliases Lib "PbSlice200.dll" Alias "pbsProjectStandardizeExportAliases"( exportList As String )
Declare Function pbsProjectToBBCode Lib "PbSlice200.dll" Alias "pbsProjectToBBCode"( ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long, ByVal includeMonoFormat As Long, ByVal includeColorFormat As Long )
Declare Sub      pbsProjectToCasedText Lib "PbSlice200.dll" Alias "pbsProjectToCasedText"( ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long )
Declare Function pbsProjectToHtml Lib "PbSlice200.dll" Alias "pbsProjectToHtml"( ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long, ByVal includeMonoFormat As Long, ByVal includeColorFormat As Long )
Declare Sub      pbsProjectTrimWhitespace Lib "PbSlice200.dll" Alias "pbsProjectTrimWhitespace"()
Declare Sub      pbsSnippetActivate Lib "PbSlice200.dll" Alias "pbsSnippetActivate"( snippetName As String )
Declare Sub      pbsSnippetAddDebugAliases Lib "PbSlice200.dll" Alias "pbsSnippetAddDebugAliases"()
Declare Sub      pbsSnippetAddText Lib "PbSlice200.dll" Alias "pbsSnippetAddText"( value As String )
Declare Function pbsSnippetBuildDllInclude Lib "PbSlice200.dll" Alias "pbsSnippetBuildDllInclude"( libName As String )
Declare Function pbsSnippetBuildSllInclude Lib "PbSlice200.dll" Alias "pbsSnippetBuildSllInclude"( libName As String )
Declare Sub      pbsSnippetClear Lib "PbSlice200.dll" Alias "pbsSnippetClear"()
Declare Sub      pbsSnippetClose Lib "PbSlice200.dll" Alias "pbsSnippetClose"( snippetName As String )
Declare Function pbsSnippetCount Lib "PbSlice200.dll" Alias "pbsSnippetCount"()
Declare Sub      pbsSnippetDeclareClear Lib "PbSlice200.dll" Alias "pbsSnippetDeclareClear"()
Declare Function pbsSnippetDeclareFindName Lib "PbSlice200.dll" Alias "pbsSnippetDeclareFindName"( ByVal start As Long, value As String, ByVal declType As Long )
Declare Function pbsSnippetDeclareGetClassInterfaces Lib "PbSlice200.dll" Alias "pbsSnippetDeclareGetClassInterfaces"( ByVal index As Long)
Declare Function pbsSnippetDeclareGetClassMethods Lib "PbSlice200.dll" Alias "pbsSnippetDeclareGetClassMethods"( ByVal index As Long )
Declare Function pbsSnippetDeclareGetClassOutline Lib "PbSlice200.dll" Alias "pbsSnippetDeclareGetClassOutline"( ByVal index As Long )
Declare Function pbsSnippetDeclareGetCount Lib "PbSlice200.dll" Alias "pbsSnippetDeclareGetCount"()
Declare Function pbsSnippetDeclareGetDeclarationName Lib "PbSlice200.dll" Alias "pbsSnippetDeclareGetDeclarationName"( ByVal index As Long )
Declare Function pbsSnippetDeclareGetDefinition Lib "PbSlice200.dll" Alias "pbsSnippetDeclareGetDefinition"( ByVal index As Long )
Declare Function pbsSnippetDeclareGetFileNumber Lib "PbSlice200.dll" Alias "pbsSnippetDeclareGetFileNumber"()
Declare Function pbsSnippetDeclareGetInstanceVariables Lib "PbSlice200.dll" Alias "pbsSnippetDeclareGetInstanceVariables"( ByVal index As Long )
Declare Function pbsSnippetDeclareGetInterfaces Lib "PbSlice200.dll" Alias "pbsSnippetDeclareGetInterfaces"()
Declare Sub      pbsSnippetDeclareGetItem Lib "PbSlice200.dll" Alias "pbsSnippetDeclareGetItem"( ByVal index As Long, value As TDecl )
Declare Function pbsSnippetDeclareGetName Lib "PbSlice200.dll" Alias "pbsSnippetDeclareGetName"( ByVal index As Long )
Declare Function pbsSnippetDeclareGetNames Lib "PbSlice200.dll" Alias "pbsSnippetDeclareGetNames"( ByVal declType As Long )
Declare Function pbsSnippetDeclareGetSignature Lib "PbSlice200.dll" Alias "pbsSnippetDeclareGetSignature"( ByVal index As Long )
Declare Sub      pbsSnippetDeclareItem Lib "PbSlice200.dll" Alias "pbsSnippetDeclareItem"( ByVal index As Long, value As TDecl )
Declare Sub      pbsSnippetDeclareScan Lib "PbSlice200.dll" Alias "pbsSnippetDeclareScan"( Optional ByVal pClear As Long )
Declare Sub      pbsSnippetDeclareSetSnippet Lib "PbSlice200.dll" Alias "pbsSnippetDeclareSetSnippet"( snippetName As String )
Declare Sub      pbsSnippetDeleteChars Lib "PbSlice200.dll" Alias "pbsSnippetDeleteChars"( ByVal lineNumber As Long, ByVal index As Long, ByVal cnt As Long )
Declare Sub      pbsSnippetDeleteText Lib "PbSlice200.dll" Alias "pbsSnippetDeleteText"( ByVal lineNumber As Long, cnt As Long )
Declare Sub      pbsSnippetExpandTabs Lib "PbSlice200.dll" Alias "pbsSnippetExpandTabs"( Optional ByVal tabSize As Long )
Declare Function pbsSnippetFindExports Lib "PbSlice200.dll" Alias "pbsSnippetFindExports"()
Declare Function pbsSnippetGetActiveName Lib "PbSlice200.dll" Alias "pbsSnippetGetActiveName"()
Declare Function pbsSnippetGetFilename Lib "PbSlice200.dll" Alias "pbsSnippetGetFilename"()
Declare Function pbsSnippetGetIndex Lib "PbSlice200.dll" Alias "pbsSnippetGetIndex"()
Declare Function pbsSnippetGetInterfaceClasses Lib "PbSlice200.dll" Alias "pbsSnippetGetInterfaceClasses"( interfaceName As String )
Declare Function pbsSnippetGetInterfaceDeclaration Lib "PbSlice200.dll" Alias "pbsSnippetGetInterfaceDeclaration"( ByVal index As Long, constants As String )
Declare Function pbsSnippetGetInterfaceRoutines Lib "PbSlice200.dll" Alias "pbsSnippetGetInterfaceRoutines"( ByVal index As Long )
Declare Function pbsSnippetGetInterfaceStub Lib "PbSlice200.dll" Alias "pbsSnippetGetInterfaceStub"( ByVal index As Long )
Declare Function pbsSnippetGetLine Lib "PbSlice200.dll" Alias "pbsSnippetGetLine"( ByVal lineNumber As Long )
Declare Function pbsSnippetGetLines Lib "PbSlice200.dll" Alias "pbsSnippetGetLines"( ByVal lineNumber As Long, cnt As Long )
Declare Function pbsSnippetGetText Lib "PbSlice200.dll" Alias "pbsSnippetGetText"()
Declare Sub      pbsSnippetIndent Lib "PbSlice200.dll" Alias "pbsSnippetIndent"()
Declare Sub      pbsSnippetInsertChars Lib "PbSlice200.dll" Alias "pbsSnippetInsertChars"( ByVal lineNumber As Long, ByVal index As Long, insertString As String )
Declare Sub      pbsSnippetInsertClass Lib "PbSlice200.dll" Alias "pbsSnippetInsertClass"( ByVal lineNumber As Long, className As String, Optional ByVal addGUID As Long, Optional ByVal isCommon As Long )
Declare Sub      pbsSnippetInsertInterface Lib "PbSlice200.dll" Alias "pbsSnippetInsertInterface"( ByVal lineNumber As Long, interfaceName As String, inheritFrom As String, Optional ByVal addGUID As Long )
Declare Sub      pbsSnippetInsertText Lib "PbSlice200.dll" Alias "pbsSnippetInsertText"( ByVal lineNumber As Long, value As String )
Declare Function pbsSnippetLineCount Lib "PbSlice200.dll" Alias "pbsSnippetLineCount"()
Declare Sub      pbsSnippetLoad Lib "PbSlice200.dll" Alias "pbsSnippetLoad"( filename As String )
Declare Sub      pbsSnippetManageBlankLines Lib "PbSlice200.dll" Alias "pbsSnippetManageBlankLines"()
Declare Function pbsSnippetNames Lib "PbSlice200.dll" Alias "pbsSnippetNames"()
Declare Sub      pbsSnippetNew Lib "PbSlice200.dll" Alias "pbsSnippetNew"( snippetName As String )
Declare Sub      pbsSnippetReload Lib "PbSlice200.dll" Alias "pbsSnippetReload"()
Declare Sub      pbsSnippetRemoveAliases Lib "PbSlice200.dll" Alias "pbsSnippetRemoveAliases"()
Declare Sub      pbsSnippetRemoveContinuation Lib "PbSlice200.dll" Alias "pbsSnippetRemoveContinuation"()
Declare Sub      pbsSnippetReplaceChars Lib "PbSlice200.dll" Alias "pbsSnippetReplaceChars"( ByVal lineNumber As Long, ByVal index As Long, ByVal cnt As Long, insertString As String )
Declare Sub      pbsSnippetReplaceText Lib "PbSlice200.dll" Alias "pbsSnippetReplaceText"( ByVal lineNumber As Long, ByVal cnt As Long, value As String )
Declare Sub      pbsSnippetReplaceToken Lib "PbSlice200.dll" Alias "pbsSnippetReplaceToken"( originalToken As String, newToken As String )
Declare Sub      pbsSnippetSave Lib "PbSlice200.dll" Alias "pbsSnippetSave"( Optional filename As String )
Declare Sub      pbsSnippetSetFilename Lib "PbSlice200.dll" Alias "pbsSnippetSetFilename"( value As String )
Declare Sub      pbsSnippetSetIndex Lib "PbSlice200.dll" Alias "pbsSnippetSetIndex"( ByVal index As Long )
Declare Sub      pbsSnippetSetLine Lib "PbSlice200.dll" Alias "pbsSnippetSetLine"( ByVal lineNumber As Long, value As String )
Declare Sub      pbsSnippetSetLineCount Lib "PbSlice200.dll" Alias "pbsSnippetSetLineCount"( ByVal value As Long )
Declare Sub      pbsSnippetSetText Lib "PbSlice200.dll" Alias "pbsSnippetSetText"( value As String )
Declare Function pbsSnippetSizeInBytes Lib "PbSlice200.dll" Alias "pbsSnippetSizeInBytes"()
Declare Sub      pbsSnippetStandardizeExportAliases Lib "PbSlice200.dll" Alias "pbsSnippetStandardizeExportAliases"( exportList As String )
Declare Function pbsSnippetToBBCode Lib "PbSlice200.dll" Alias "pbsSnippetToBBCode"( title As String, ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long, ByVal includeMonoFormat As Long,
																		ByVal includeColorFormat As Long )
Declare Sub      pbsSnippetToCasedText Lib "PbSlice200.dll" Alias "pbsSnippetToCasedText"( ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long )
Declare Function pbsSnippetToHtml Lib "PbSlice200.dll" Alias "pbsSnippetToHtml"( title As String, ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long, ByVal includeMonoFormat As Long, ByVal includeColorFormat As Long )
Declare Sub      pbsSnippetTrimWhitespace Lib "PbSlice200.dll" Alias "pbsSnippetTrimWhitespace"()
Declare Function pbsStringCompressSignature Lib "PbSlice200.dll" Alias "pbsStringCompressSignature"( value As String )
Declare Function pbsStringExpandTabs Lib "PbSlice200.dll" Alias "pbsStringExpandTabs"( value As String, ByVal tabSize As Long )
Declare Function pbsStringTrimWhitespace Lib "PbSlice200.dll" Alias "pbsStringTrimWhitespace"( value As String )
Declare Function pbsTokenCount Lib "PbSlice200.dll" Alias "pbsTokenCount"()
Declare Function pbsTokenGetToken Lib "PbSlice200.dll" Alias "pbsTokenGetToken"( ByVal i As Long, pToken As Dword, pColumn As Long, pTokenType As Long )
Declare Sub      pbsTokenSetFileLine Lib "PbSlice200.dll" Alias "pbsTokenSetFileLine"( ByVal fileNumber As Long, ByVal lineNumber As Long )
Declare Sub      pbsTokenSetLine Lib "PbSlice200.dll" Alias "pbsTokenSetLine"( ByVal value As String )
