' File based API routines
' These routines work with a single file in a project rather than the entire project.
' The routines work similar to snippets in that you activate a file
' subsequent calls use the last activated file.
'

Sub pbsFileActivate Alias "pbsFileActivate"( ByVal fileNumber As Long ) Export
	If Not init_ Then Call Initialize()
	If fileNumber < 0 Or fileNumber >= project_.Count Then Exit Sub
	activeFilenumber_ = fileNumber
	sourceFile_ = project_.Value( fileNumber )
End Sub

Function pbsFileIndexFilename Alias "pbsFileIndexFilename"( value As String ) Export As Long
	Local fullName As String
	If Not init_ Then Call Initialize()
	fullName = value
	Call GetFullPath( fullName )
	Function = project_.Index( fullname )
End Function

Sub pbsFileLoad Alias "pbsFileLoad"( filename As String ) Export
	Local sf As iSourceFile
	Local fullName As String
	Local isExternal As Long
	Local i As Long

	If Not init_ Then Call Initialize()
	fullName = filename
	Call GetFullPath( fullName )
	i = project_.Index( fullName )
	If i>=0 Then
		pbsFileActivate( i )
		Exit Sub
	End If

	isExternal = pbsFilenameIsExternal( fullName )
	Call NewSourceFile( sf, fullName, isExternal )
	sf.Reload()
	pbsFileActivate( project_.Add( fullName, sf ) )
End Sub

Sub pbsFileNew Alias "pbsFileNew"( filename As String, Optional overwrite As Long ) Export
	Local sf As iSourceFile
	Local fullName As String
	Local i, isExternal As Long

	If Not init_ Then Call Initialize()
	fullName = filename
	Call GetFullPath( fullName )
	i = project_.Index( fullName )
	If i>=1 Then
		pbsFileActivate( i )
		Exit Sub
	End If
	If IsMissing( overwrite ) Or IsFalse overwrite Then
		If IsFile( fullName ) Then Exit Sub:          ' File exists, can't be new
	End If
	isExternal = pbsFilenameIsExternal( fullName )
	Call NewSourceFile( sf, fullName, isExternal )
	pbsFileActivate( project_.Add( fullName, sf ) )
	sf.Text = ""
	sf.Save
End Sub

Function pbsFileGetIsExternal Alias "pbsFileGetIsExternal"() Export As Long
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Function
	Function = sourceFile_.IsExternal
End Function

Sub pbsFileSetIsExternal Alias "pbsFileSetIsExternal"( ByVal value As Long ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	sourceFile_.IsExternal = value
End Sub


Sub pbsFileClose Alias "pbsFileClose"( ByVal fileNumber As Long ) Export
	Local i As Long
	If Not init_ Then Call Initialize()
	If fileNumber <0 Or fileNumber >= project_.Count Then Exit Sub
	i = sfFileClose( project_, project_.Key( fileNumber ) )
	If i = activeSnippetIndex_ Then
		If i >= project_.Count Then Decr activeFilenumber_
		If i >=0 Then pbsFileActivate( activeFilenumber_ )
	Else
		If i>-1 And i<activeFilenumber_ Then Decr activeFilenumber_
	End If
End Sub

Function pbsFileGetFilenumber Alias "pbsFileGetFilenumber"() Export As Long
	Function = activeFilenumber_
End Function

Sub pbsFileReload Alias "pbsFileReload"() Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	sourceFile_.Reload()
End Sub

Sub pbsFileSave Alias "pbsFileSave"( Optional newFilename As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	If IsMissing( newFilename ) Then
		sourceFile_.Save()
	Else
		sourceFile_.Save( newFilename )
	End If
End Sub

Function pbsFilenameIsExternal Alias "pbsFilenameIsExternal"( filename As String ) Export As Long
	Local curPath As String

	If Not init_ Then Call Initialize()
	Call pbsProjectGetBasePath( curPath )
	Function = ( curPath<>"" And LCase$( Left$( filename, Len(curPath) ) ) <> LCase$( curPath ) )
End Function

Function pbsFileGetFilename Alias "pbsFileGetFilename"() Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Function
	Function = sourceFile_.Filename
End Function

Sub pbsFileSetFilename Alias "pbsFileSetFilename"( value As String ) Export
	Local fullName As String
	Local prj As iStringSourcefileMap

	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	fullName = value
	Call GetFullPath( fullName )
	sourceFile_.Filename = fullName
	Call DupStringSourcefileMap( prj, project_ )
	project_ = Nothing
	project_ = prj
	prj = Nothing
End Sub

Sub pbsFileSetText Alias "pbsFileSetText"( value As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	sourceFile_.Text = value
End Sub

Function pbsFileGetText Alias "pbsFileGetText"() Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Function
	Function = sourceFile_.Text
End Function

Function pbsFileByteSize Alias "pbsFileByteSize"() Export As Long
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Function
	Function = sourceFile_.SizeInBytes
End Function

Function pbsFileGetLineCount Alias "pbsFileGetLineCount"() Export As Long
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Function
	Function = sourceFile_.Count
End Function

Sub pbsFileSetLineCount Alias "pbsFileSetLineCount"( ByVal value As Long ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	sourceFile_.Count = value
End Sub

Function pbsFileShortname Alias "pbsFileShortname"( filename As String ) Export As String
	Local fullName As String
	Local basePath As String
	Local i As Long

	If Not init_ Then Call Initialize()
	fullName = filename
	Call GetFullPath( fullName )
	Call pbsProjectGetBasePath( basePath )

	basePath = LCase$( basePath )
	If LCase$( Left$( filename, Len(basePath) ) ) = basePath Then
		Function = Mid$( filename, Len(basePath)+1 )
		Exit Function
	End If

	For i = 0 To includePathCount_-1
		If LCase$( Left$( filename, Len( includePaths_(i) ) ) ) = LCase$( includePaths_(i) ) Then
			Function = Mid$( filename, Len( includePaths_(i) )+1 )
			Exit Function
		End If
	Next
End Function

Sub pbsFileAddText Alias "pbsFileAddText"( insertLines As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	sourceFile_.AddText( insertLines )
End Sub

Sub pbsFileInsertText Alias "pbsFileInsertText"( ByVal insertAt As Long, insertLines As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	sourceFile_.InsertText( insertAt, insertLines )
End Sub

Sub pbsFileDeleteText Alias "pbsFileDeleteText"( ByVal startAt As Long, ByVal cnt As Long ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	sourceFile_.DeleteText( startAt, cnt )
End Sub

Sub pbsFileReplaceText Alias "pbsFileReplaceText"( ByVal startAt As Long, ByVal cnt As Long, insertLines As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	sourceFile_.ReplaceText( startAt, cnt, insertLines )
End Sub

Sub pbsFileInsertChars Alias "pbsFileInsertChars"( ByVal lineNumber As Long, ByVal index As Long, insertString As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	sourceFile_.InsertChars( lineNumber, index, insertString )
End Sub

Sub pbsFileDeleteChars Alias "pbsFileDeleteChars"( ByVal lineNumber As Long, ByVal index As Long, ByVal cnt As Long ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	sourceFile_.DeleteChars( lineNumber, index, cnt )
End Sub

Sub pbsFileReplaceChars Alias "pbsFileReplaceChars"( ByVal lineNumber As Long, ByVal index As Long, ByVal cnt As Long, insertString As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	sourceFile_.ReplaceChars( lineNumber, index, cnt, insertString )
End Sub

Function pbsFileGetLine Alias "pbsFileGetLine"( ByVal lineNumber As Long ) Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Function
	Function = sourceFile_.Line( lineNumber )
End Function

Sub pbsFileSetLine Alias "pbsFileSetLine"( ByVal lineNumber As Long, value As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	sourceFile_.Line( lineNumber ) = value
End Sub

Function pbsFileGetLines Alias "pbsFileGetLines"( ByVal lineNumber As Long, ByVal cnt As Long ) Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Function
	Function = sourceFile_.Lines( lineNumber, cnt )
End Function

Sub pbsFileGetTextArray Alias "pbsFileGetTextArray"( ByVal lineNumber As Long, ByVal cnt As Long, values() As String ) Export
	Local i As Long
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub

	If lineNumber<0 Then cnt += lineNumber: lineNumber = 0
	If lineNumber >= sourceFile_.Count Or cnt<0 Then Erase values(): Exit Sub
	If lineNumber+cnt >= sourceFile_.Count Then cnt = sourceFile_.Count - lineNumber
	ReDim values( cnt-1 )
	For i=0 To cnt-1
		values(i) = sourceFile_.Line( lineNumber+i )
	Next
End Sub

Function pbsFileToBBCode Alias "pbsFileToBBCode"( ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long, ByVal includeMonoFormat As Long, ByVal includeColorFormat As Long ) Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Function
	Function = FileToBBCode( sourceFile_, tabSize, keywordCasing, singleCaseIdentifier, includeMonoFormat, includeColorFormat )
End Function

Function pbsFileToHtml Alias "pbsFileToHtml"( ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long, ByVal includeMonoFormat As Long, ByVal includeColorFormat As Long ) Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Function
	Function = FileToHtml( sourceFile_, tabSize, keywordCasing, singleCaseIdentifier, includeMonoFormat, includeColorFormat )
End Function

Sub pbsFileCasedText Alias "pbsFileCasedText"( ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	Call sfToCasedText( sourceFile_, tabSize, keywordCasing, singleCaseIdentifier )
End Sub

Sub pbsFileReplaceToken Alias "pbsFileReplaceToken"( originalToken As String, newToken As String ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	Call sfReplaceToken( sourceFile_, originalToken, newToken )
End Sub

Sub pbsFileIndent Alias "pbsFileIndent"() Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	indenter_.Indent( sourceFile_ )
End Sub

Sub pbsFileExpandTabs Alias "pbsFileExpandTabs" ( Optional ByVal tabSize As Long ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	If tabSize < 1 Then tabSize = tabSize_
	If tabSize < 1 Then Exit Sub
	Call sfExpandTabs( sourceFile_, tabSize )
End Sub

Sub pbsFileRemoveContinuation Alias "pbsFileRemoveContinuation" () Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	sfRemoveContinuation( sourceFile_ )
End Sub

Sub pbsFileTrimWhitespace Alias "pbsFileTrimWhitespace"() Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	Call sfTrimWhitespace( sourceFile_ )
End Sub

Function pbsFileFindExports Alias "pbsFileFindExports"() Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Function
	Function = sfFindExports( sourceFile_ )
End Function

Sub pbsFileInsertClass Alias "pbsFileInsertClass"( ByVal lineNumber As Long, className As String, Optional ByVal addGUID As Long, Optional ByVal isCommon As Long ) Export
	Local g As String

	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub

	If addGUID Then g = " Guid$(" + Wrap$( LCase$( GuidTxt$( Guid$ ) ), $Dq, $Dq ) + ")"
	If isCommon Then g += " Common"
	sourceFile_.InsertText( lineNumber, Build$( "Class ", className, g, $CrLf, "End Class" ) )
End Sub

Sub pbsFileInsertInterface Alias "pbsFileInsertInterface"( ByVal lineNumber As Long, interfaceName As String, inheritFrom As String, Optional ByVal addGUID As Long ) Export
	Local g As String

	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub

	If addGUID Then g = " Guid$(" + Wrap$( LCase$( GuidTxt$( Guid$ ) ), $Dq, $Dq ) + ")"
	sourceFile_.InsertText( lineNumber, Build$( "  Interface ", interfaceName, g, $CrLf, "    Inherit ", inheritFrom, $CrLf, "  End Interface" ) )
End Sub

' =========================================================================
' The are separate from other types of Declare routines and cumulative unless cleared
' Most routines assume a file has been scanned
Function pbsFileDeclareGetAllInterfaces Alias "pbsFileDeclareGetAllInterfaces"() Export As String
	Function = sfDeclareGetInterfaces( projectDeclares_ )
End Function

Sub pbsFileDeclareScan Alias "pbsFileDeclareScan"( Optional ByVal pClear As Long  ) Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	projectDeclares_.Scan( sourceFile_, activeFilenumber_, pClear )
End Sub

Function pbsFileDeclareGetNames Alias "pbsFileDeclareGetNames"( ByVal tp As Long ) Export As String
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Function
	Function = projectDeclares_.GetNames( tp, activeFilenumber_ )
End Function

' =========================================================================

Sub pbsFileManageBlankLines Alias "pbsFileManageBlankLines"() Export
	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Sub
	sfManageBlankLines( sourceFile_ )
End Sub

Function pbsFileBuildDLLInclude Alias "pbsFileBuildDLLInclude"() Export As String
	Local exportList, routines, interfaces, constants, enums, types, unions As String
	Local libName As String

	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Function

	libName = PathScan$( Name, project_.Value( 0 ).FileName )
	libName += ".dll"

	exportList = sfFindExports( sourceFile_ )
	projectDeclares_.Scan( sourceFile_, activeFilenumber_, -1 )
	Call projectDeclares_.GetDLLDeclaration( libName, exportList, routines, interfaces, constants, enums, types, unions )
	Function = BuildInc( "Include file for DLL users of " + libName, routines, interfaces, constants, enums, types, unions )
End Function

Function pbsFileBuildSLLInclude Alias "pbsFileBuildSLLInclude"() Export As String
	Local exportList, routines, interfaces, constants, enums, types, unions As String
	Local libName As String

	If Not init_ Then Call Initialize()
	If IsNothing( sourceFile_ ) Then Exit Function

	libName = PathScan$( Name, project_.Value( 0 ).FileName )
	libName += ".dll"

	exportList = sfFindExports( sourceFile_ )
	projectDeclares_.Scan( sourceFile_, activeFilenumber_, -1 )
	Call projectDeclares_.GetSLLDeclaration( exportList, routines, interfaces, constants, enums, types, unions )
	Function = BuildInc( "Include file for DLL users of " + libName, routines, interfaces, constants, enums, types, unions )
End Function
