' Project API routines

Sub Initialize()
	If init_ Then Exit Sub
	Try
		tabSize_ = 2
		Call NewStringSourcefileMap( project_, 400, 50 )
		Call NewStringSourcefileMap( snippets_, 400, 10 )

		Call NewDeclarations( projectDeclares_ )
		Call NewDeclarations( snippetDeclares_ )

		Call NewStringBag( keywords_, 2000 )
		Call NewLineTokenizer( tokenizer_, keywords_ )
		Call NewIndent( indenter_ )
		activeSnippetIndex_ = -1
		activeFileNumber_ = -1
		init_ = -1
	Catch
	End Try
End Sub

Sub pbsProjectGetBasePath Alias "pbsProjectGetBasePath"( projectPath As String ) Export
	If Not init_ Then Call Initialize()

	If project_.Count Then
		projectPath = PathName$( Path, project_.Value( 0 ).Filename )
		If Right$( projectPath, 1 )<>"\" Then projectPath+="\"
	Else
		projectPath = ""
	End If
End Sub

Sub pbsProjectSetBasePath Alias "pbsProjectSetBasePath"( projectPath As String ) Export
	Local curPath, wPath As String
	Local i As Long
	Local sf As iSourceFile

	If Not init_ Then Call Initialize()

	Call pbsProjectGetBasePath( curPath )
	curPath = LCase$( curPath )

	wPath = projectPath
	If Right$( wPath, 1 )<>"\" Then wPath += "\"

	If curPath = LCase$( wPath ) Then Exit Sub: ' Already there

	If EnsurePath( projectPath ) Then Exit Sub: ' Error

	' Change all base paths for non-external files
	For i=0 To project_.Count-1
		sf = project_.Value( i )
		If LCase$( Left$( sf.Filename, Len( curPath ) ) ) = curPath Then
			sf.Filename = wPath + Mid$( sf.Filename, Len( curPath )+1 )
		End If
	Next
End Sub

Sub pbsProjectLoad Alias "pbsProjectLoad"( mainFile As String ) Export
	#Debug Code Off
	Local files As iStringStack
	Local filename As String
	Local sf As iSourceFile
	Local basePath As String
	Local isExternal As Long
	Local r As Long
	Local value As String
	Local includeFile As String

	If Not init_ Then Call Initialize()
	Call pbsProjectClose(): ' Ensure any current project is closed

	Call NewStringStack( files )
	files.Push( mainFile )

	Do While files.Count
		filename = files.Pop()
		If IsFalse GetFullPath( filename ) Then Iterate Loop: ' File not found
		If project_.Contains( filename ) Then Iterate Loop: ' Already loaded
		If basePath="" And project_.Count>0 Then
			Call pbsProjectGetBasePath( basePath )
			basePath = LCase$( basePath )
		End If
		isExternal = pbsFilenameIsExternal( filename )
		Call NewSourceFile( sf, filename, isExternal )
		If IsInterface( sf, iSourceFile ) Then
			sf.Reload()
			project_.Add( filename, sf )

			For r=0 To sf.Count-1
				value = sf.Line( r )
				If FindInclude( value, includeFile ) Then
					files.Push( includeFile )
				End If
			Next
		End If
		sf = Nothing
	Loop
	#Debug Code On
End Sub

Sub pbsProjectSave Alias "pbsProjectSave"() Export
	Local i As Long
	Local sf As iSourceFile

	If Not init_ Then Call Initialize()
	For i = 0 To project_.Count-1
		sf = project_.Value( i )
		If IsFalse sf.IsExternal Then
			sf.Save()
		End If
	Next
End Sub

Sub pbsProjectNew Alias "pbsProjectNew"( mainFile As String ) Export
	Local f As Long
	Local sf As iSourceFile

	If Not init_ Then Call Initialize()
	If Not IsFile( mainFile ) Then
		Call EnsureFileAndPath( mainFile )
	End If

	Call pbsProjectClose(): ' Ensure any current project is closed
	Call NewSourceFile( sf, mainFile, 0 )
	If IsInterface( sf, iSourceFile ) Then
		sf.Reload()
		project_.Add( mainfile, sf )
	End If
End Sub

Sub pbsProjectClose Alias "pbsProjectClose"() Export
	If Not init_ Then Call Initialize()
	project_.Clear()
End Sub

Sub pbsProjectReload Alias "pbsProjectReload"() Export
	Local i, c As Long

	If Not init_ Then Call Initialize()
	c = project_.Count - 1
	For i=0 To c
		project_.Value( i ).Reload
	Next
End Sub

Function pbsProjectGetFilename Alias "pbsProjectGetFilename"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	If index>=0 And index< project_.Count Then
		Function = project_.Value( index ).Filename
	Else
		Function = ""
	End If
End Function

Sub pbsProjectSetFilename Alias "pbsProjectSetFilename"( ByVal index As Long, filename As String ) Export
	If Not init_ Then Call Initialize()
	If index>=0 And index< project_.Count Then
		project_.Value( index ).Filename = filename
	End If
End Sub

Function pbsProjectGetFileNumber Alias "pbsProjectGetFileNumber"( filename As String ) Export As Long
	Local fullName As String
	If Not init_ Then Call Initialize()
	fullName = filename
	Call GetFullPath( fullName )
	Function = project_.Index( fullName )
End Function

Function pbsProjectHasFile Alias "pbsProjectHasFile"( filename As String ) Export As Long
	Local fullName As String
	If Not init_ Then Call Initialize()
	fullName = filename
	Call GetFullPath( fullName )
	Function = ( project_.Contains( fullName ) )
End Function

Function pbsProjectByteCount Alias "pbsProjectByteCount"() Export As Long
	Local i, c, sz As Long

	If Not init_ Then Call Initialize()
	c = project_.Count-1
	For i=0 To c
		sz += project_.Value(i).SizeInBytes
	Next
	Function = sz
End Function

Function pbsProjectLineCount Alias "pbsProjectLineCount"() Export As Long
	Local i, c, sz As Long

	If Not init_ Then Call Initialize()
	c = project_.Count-1
	For i=0 To c
		sz += project_.Value(i).Count
	Next
	Function = sz
End Function

Function pbsProjectFileCount Alias "pbsProjectFileCount"() Export As Long
	If Not init_ Then Call Initialize()
	Function = project_.Count
End Function

Sub pbsProjectSetIncludePaths Alias "pbsProjectSetIncludePaths"( paths As String ) Export
	Local i, j As Long

	If Not init_ Then Call Initialize()

	includePathCount_ = StrSplit( paths, includePaths_(), ";" )

	' Remove any invalid paths
	For i = includePathCount_-1 To 0 Step -1
		includePaths_(i) = Trim$( includePaths_(i) ): ' Remove leading/trailing spaces
		If Right$( includePaths_(i), 1 )<>"\" Then includePaths_(i) += "\": 'Ensure ends with \

		If IsFalse IsFolder( includePaths_(i) ) Or includePaths_(i)="\" Then
			Decr includePathCount_
			For j=i To includePathCount_-1
				Swap includePaths_(j), includePaths_(j+1)
			Next
		End If
	Next
	ReDim Preserve includePaths_( includePathCount_-1 )
End Sub

Sub pbsProjectGetIncludePaths Alias "pbsProjectGetIncludePaths"( paths As String ) Export
	If Not init_ Then Call Initialize()
	paths = Join$( includePaths_(), ";" )
End Sub

Sub pbsProjectCloseFile Alias "pbsProjectCloseFile"( ByVal fileNumber As Long ) Export
	Local prj As iStringSourcefileMap
	Local sf As iSourceFile
	Local filename As String
	Local i As Long

	If Not init_ Then Call Initialize()
	If fileNumber<0 Or fileNumber >= project_.Count Then Exit Sub
	sf = project_.Value( fileNumber )
	If IsFalse IsInterface( sf, iSourceFile ) Then Exit Sub
	filename = LCase$( sf.Filename )
	sf = Nothing

	Call NewStringSourcefileMap( prj, 400, 50 )
	For i=0 To project_.Count-1
		If i<>fileNumber Then
			sf = project_.Value( i )
			prj.Add( sf.filename, sf )
			sf = Nothing
		End If
	Next
	Call pbsProjectClose()
	project_ = prj
	prj = Nothing
End Sub

Sub pbsProjectSetTabsize Alias "pbsProjectSetTabsize"( ByVal tabSize As Long ) Export
	If Not init_ Then Call Initialize()
	If tabSize>-1 Then
		tabSize_ = tabSize
	End If
End Sub

Function pbsProjectGetTabsize Alias "pbsProjectGetTabsize"() Export As Long
	If Not init_ Then Call Initialize()
	Function = tabSize_
End Function

Function pbsProjectToBBCode Alias "pbsProjectToBBCode"( ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long, ByVal includeMonoFormat As Long, ByVal includeColorFormat As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = ProjectToBBCode( tabSize, keywordCasing, singleCaseIdentifier, includeMonoFormat, includeColorFormat )
End Function

Function pbsProjectToHtml Alias "pbsProjectToHtml"( ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long, ByVal includeMonoFormat As Long, ByVal includeColorFormat As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = ProjectToHtml( tabSize, keywordCasing, singleCaseIdentifier, includeMonoFormat, includeColorFormat )
End Function

Sub pbsProjectToCasedText Alias "pbsProjectToCasedText"( ByVal tabSize As Long, ByVal keywordCasing As Long, ByVal singleCaseIdentifier As Long ) Export
	Local i As Long
	Local sf As iSourceFile

	If Not init_ Then Call Initialize()
	For i = 0 To project_.Count-1
		If Not project_.Value(i).IsExternal Then
			sf = project_.Value(i)
			sfToCasedText( sf, tabSize, keywordCasing, singleCaseIdentifier )
		End If
	Next
End Sub

Sub pbsProjectReplaceToken Alias "pbsProjectReplaceToken"( originalToken As String, newToken As String ) Export
	Local i As Long
	Local sf As iSourceFile

	If Not init_ Then Call Initialize()
	For i = 0 To project_.Count-1
		If Not project_.Value( i ).IsExternal Then
			sf = project_.Value(i)
			Call sfReplaceToken( sf, originalToken, newToken )
		End If
	Next
End Sub

Sub pbsProjectIndent Alias "pbsProjectIndent"() Export
	Local i As Long
	Local sf As iSourceFile

	If Not init_ Then Call Initialize()
	For i = 0 To project_.Count-1
		If Not project_.Value( i ).IsExternal Then
			sf = project_.Value(i)
			indenter_.Indent( sf )
			project_.Value(i) = sf
		End If
	Next
End Sub

Sub pbsProjectExpandTabs Alias "pbsProjectExpandTabs" ( Optional ByVal tabSize As Long ) Export
	Local i As Long
	Local sf As iSourceFile

	If Not init_ Then Call Initialize()
	If tabSize < 1 Then tabSize = tabSize_
	If tabSize < 1 Then Exit Sub
	For i = 0 To project_.Count-1
		If Not project_.Value( i ).IsExternal Then
			sf = project_.Value(i)
			Call sfExpandTabs( sf, tabSize )
		End If
	Next
End Sub

Sub pbsProjectRemoveContinuation Alias "pbsProjectRemoveContinuation" () Export
	Local i As Long
	Local sf As iSourceFile

	If Not init_ Then Call Initialize()
	For i = 0 To project_.Count-1
		If Not project_.Value( i ).IsExternal Then
			sf =project_.Value(i)
			sfRemoveContinuation( sf )
			project_.Value(i) = sf
		End If
	Next
End Sub

Sub pbsProjectTrimWhitespace Alias "pbsProjectTrimWhitespace" () Export
	Local i As Long
	Local sf As iSourceFile

	If Not init_ Then Call Initialize()
	For i = 0 To project_.Count-1
		If Not project_.Value( i ).IsExternal Then
			sf = project_.Value(i)
			Call sfTrimWhitespace( sf )
		End If
	Next
End Sub

' =========================================================================
' These are separate from all other Declare routines
Function pbsProjectDeclareGetAllInterfaces Alias "pbsProjectDeclareGetAllInterfaces"() Export As String
	Local i, j, c As Long
	Local decl As TDecl
	Local names As iStringBag
	Local list As String
	Local v() As String

	If Not init_ Then Call Initialize()
	Call NewStringBag( names, 5000, 100 )

	For i=0 To projectDeclares_.Count-1
		decl = projectDeclares_.Item( i )
		If decl.tDecltype = %DeclarationTypes.eClass Then
			list = projectDeclares_.GetClassInterfaces( i )
			c = StrSplit( list, v(), $CrLf )
			For j=0 To c-1
				If v(j)<>"" Then
					names.Add( v(j) )
				End If
			Next
		End If
	Next
	ReDim v( names.Count-1 )
	For i=0 To names.Count-1
		v(i) = names.Key( i )
	Next
	Array Sort v(), Collate UCase
	Function = Join$( v(), $CrLf )
End Function

Sub pbsProjectDeclareScan Alias "pbsProjectDeclareScan"() Export
	Local i As Long
	Local sf As iSourceFile

	If Not init_ Then Call Initialize()
	projectDeclares_.Clear()

	For i = 0 To project_.Count-1
		sf = project_.Value(i)
		projectDeclares_.Scan( sf, i, 0 )
	Next
End Sub

Function pbsProjectDeclareGetName Alias "pbsProjectDeclareGetName"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = projectDeclares_.Name( index )
End Function

Function pbsProjectDeclareGetDeclarationName Alias "pbsProjectDeclareGetDeclarationName"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = GetDeclarationTypeDescription( index )
End Function

Function pbsProjectDeclareCount Alias "pbsProjectDeclareCount"() Export As Long
	If Not init_ Then Call Initialize()
	Function = projectDeclares_.Count()
End Function

Sub pbsProjectDeclareClear Alias "pbsProjectDeclareClear"() Export
	If Not init_ Then Call Initialize()
	projectDeclares_.Clear()
End Sub

Sub pbsProjectDeclareGetItem Alias "pbsProjectDeclareGetItem"( ByVal index As Long, value As TDecl ) Export
	If Not init_ Then Call Initialize()
	value = projectDeclares_.Item( index )
End Sub

Function pbsProjectDeclareGetClassMethods Alias "pbsProjectDeclareGetClassMethods"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = projectDeclares_.GetClassMethods( index )
End Function

Function pbsProjectDeclareGetClassOutline Alias "pbsProjectDeclareGetClassOutline"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = projectDeclares_.GetClassOutline( index )
End Function

Function pbsProjectDeclareGetInstanceVariables Alias "pbsProjectDeclareGetInstanceVariables"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = projectDeclares_.GetInstanceVariables( index )
End Function

Function pbsProjectDeclareGetClassInterfaces Alias "pbsProjectDeclareGetClassInterfaces"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = projectDeclares_.GetClassInterfaces( index )
End Function

Function pbsProjectDeclareGetSignature Alias "pbsProjectDeclareGetSignature"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = projectDeclares_.GetSignature( index )
End Function

Function pbsProjectDeclareGetDefinition Alias "pbsProjectDeclareGetDefinition"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = projectDeclares_.GetDefinition( index )
End Function
' =========================================================================

Sub pbsProjectStandardizeExportAliases Alias "pbsProjectStandardizeExportAliases"( exportList As String ) Export
	Local i As Long
	' Assumes project has been scanned

	If Not init_ Then Call Initialize()
	i=-1
	While IterateProjects( i )
		projectDeclares_.StandardizeExportAliases( exportList )
	Wend
End Sub

Sub pbsProjectAddDebugAliases Alias "pbsProjectAddDebugAliases"() Export
	Local i As Long
	' Assumes project has been scanned

	If Not init_ Then Call Initialize()
	i=-1
	While IterateProjects( i )
		projectDeclares_.AddDebugAliases()
	Wend
End Sub

Sub pbsProjectRemoveAliases Alias "pbsProjectRemoveAliases"() Export
	Local i As Long
	Local sf As iSourceFile
	' Assumes project has been scanned

	If Not init_ Then Call Initialize()
	For i = 0 To project_.Count-1
		If Not project_.Value( i ).IsExternal Then
			sf = project_.Value(i)
			projectDeclares_.SourceFile = sf
			projectDeclares_.FileNumber = i
			projectDeclares_.RemoveAliases()
		End If
	Next
End Sub

Sub pbsProjectManageBlankLines Alias "pbsProjectManageBlankLines"() Export
	Local i As Long
	Local sf As iSourceFile

	If Not init_ Then Call Initialize()
	i=-1
	While IterateProjects( i )
		sf = project_.Value( i )
		sfManageBlankLines( sf )
		project_.Value( i ) = sf
	Wend
End Sub

Function pbsProjectDeclareGetNames Alias "pbsProjectDeclareGetNames"( ByVal declType As Long ) Export As String
	Local u As iStringBag
	Local l(), v As String
	Local i, c As Long
	Local sb As IStringBuilderA

	If Not init_ Then Call Initialize()
	v = projectDeclares_.GetNames( declType )
	c = StrSplit( v, l(), $CrLf )
	Call NewStringBag( u, c, c )
	For i=0 To c-1
		u.Add( l(i) )
	Next
	ReDim l(u.Count-1)
	sb = Class "StringBuilderA"
	For i=0 To u.Count-1
		If i Then sb.Add( $CrLf )
		sb.Add( u.Key( i ) )
	Next
	Function = sb.String
End Function

Function pbsProjectDeclareFindName Alias "pbsProjectDeclareFindName"( ByVal start As Long, value As String, ByVal declType As Long ) Export As Long
	If Not init_ Then Call Initialize()
	Function = projectDeclares_.FindName( start, value, declType )
End Function

Function pbsProjectGetInterfaceRoutines Alias "pbsProjectGetInterfaceRoutines"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = projectDeclares_.GetInterfaceRoutines( index )
End Function

Function pbsProjectGetInterfaceStub Alias "pbsProjectGetInterfaceStub"( ByVal index As Long ) Export As String
	If Not init_ Then Call Initialize()
	Function = projectDeclares_.GetInterfaceStub( index )
End Function

Function pbsProjectGetInterfaceClasses Alias "pbsProjectGetInterfaceClasses"( interfaceName As String ) Export As String
	If Not init_ Then Call Initialize()
	Function = projectDeclares_.GetInterfaceClasses( interfaceName )
End Function

Function pbsProjectFindExports Alias "pbsProjectFindExports"() Export As String
	Local i As Long
	Local sb As IStringBuilderA
	Local value As String
	Local sf As iSourceFile

	If Not init_ Then Call Initialize()
	sb = Class "StringBuilderA"

	i=-1
	While IterateProjects( i )
		sf = project_.Value(i)
		value = sfFindExports( sf )
		If sb.Len Then sb.Add( $CrLf )
		sb.Add( value )
	Wend
	Function = sb.String
End Function

Function pbsProjectBuildDLLInclude Alias "pbsProjectBuildDLLInclude"() Export As String
	Local exportList, routines, interfaces, constants, enums, types, unions As String
	Local libName As String
	Local i As Long
	' Scan should be current

	If Not init_ Then Call Initialize()
	libName = PathScan$( Name, project_.Value( 0 ).Filename ) + ".dll"
	exportList = pbsProjectFindExports()
	i=-1
	While IterateProjects( i )
		projectDeclares_.GetDLLDeclaration( libName, exportList, routines, interfaces, constants, enums, types, unions )
	Wend
	Function = BuildInc( "Include file for users of " + libName, routines, interfaces, constants, enums, types, unions )
End Function

Function pbsProjectBuildSLLInclude Alias "pbsProjectBuildSLLInclude"() Export As String
	Local exportList, routines, interfaces, constants, enums, types, unions As String
	Local libName As String
	Local i As Long
	' Scan should be current

	If Not init_ Then Call Initialize()
	libName = PathScan$( Name, project_.Value( 0 ).Filename ) + ".sll"
	exportList = pbsProjectFindExports()

	i=-1
	While IterateProjects( i )
		Call projectDeclares_.GetSLLDeclaration( exportList, routines, interfaces, constants, enums, types, unions )
	Wend
	Function = BuildInc( "Include file for users of " + libName, routines, interfaces, constants, enums, types, unions )
End Function

Function IterateProjects( i As Long ) Private As Long
	Local sf As iSourceFile

	If i>=project_.Count Then Exit Function
	While i<project_.Count
		Incr i
		If i>=project_.Count Then Exit Function
		If IsFalse project_.Value(i).IsExternal Then Exit Loop
	Wend
	sf = project_.Value(i)
	projectDeclares_.SourceFile = sf
	projectDeclares_.FileNumber = i
	Function = -1
End Function
