' All lineNumber and character indexes start with 0

Sub NewSourceFile( obj As iSourceFile, filename As String, ByVal isExternal As Long )
	obj = Class "cSourceFile"
	obj.Filename = filename
	obj.IsExternal = isExternal
End Sub

Sub DupSourceFile( obj As iSourceFile, src As iSourceFile, Optional ByVal lineNumber As Long, Optional ByVal cnt As Long )
	' If lineNumber and cnt ommited, source file is duplicated
	' if lineNumber and cnt included, only selected region is copied
	Local i As Long
	obj = Class "cSourceFile"
	obj.Filename = src.Filename
	obj.IsExternal = src.IsExternal
	If cnt=0 Then cnt = src.Count
	If lineNumber+cnt>src.Count Then cnt = src.Count - lineNumber
	If cnt<1 Then Exit Sub
	obj.Count = cnt
	For i=cnt-1 To 0 Step -1
		obj.Line(i) = src.Line(i)
	Next
End Sub

Class cSourceFile
	Instance isExternal_ As Long
	Instance lines_() As String
	Instance count_ As Long
	Instance filename_ As String
	Debug_ClassInstance

	Debug_DefineClassCreate("cSourceFile")
	Debug_DefineClassDestroy("cSourceFile")

	Interface iSourceFile
		Inherit IUnknown

		Property Get IsExternal() As Long
			Property = isExternal_
		End Property
		Property Set IsExternal( ByVal value As Long )
			isExternal_ = value
		End Property

		Property Get SizeInBytes() As Long
			Property = Len( Me.Text )
		End Property

		Property Get Count() As Long
			Property = count_
		End Property
		Property Set Count( ByVal value As Long )
			If value>0 Then
				ReDim Preserve lines_( value-1 )
				count_ = value
			Else
				count_ = 0
				Erase lines_()
			End If
		End Property

		Method GetLine( ByVal lineNumber As Long, adr As Dword, ln As Long )
			If lineNumber < 0 Or lineNumber >= count_ Then Exit Method
			adr = StrPtr( lines_( lineNumber ) )
			ln = Len( lines_( lineNumber ) )
		End Method

		Property Get Lines( ByVal lineNumber As Long, ByVal cnt As Long ) As String
			Local lns() As String
			Local i As Long

			If lineNumber<0 Or lineNumber>=count_ Then Exit Property
			If lineNumber+cnt>count_ Then cnt=count_-lineNumber
			If cnt<1 Then Exit Property

			ReDim lns(cnt-1)

			For i=0 To cnt-1
				lns(i) = lines_( lineNumber+i )
			Next
			Property = Join$( lns(), $CrLf )
		End Property

		Property Get Text() As String
			If UBound( lines_() ) >= count_ Then
				ReDim Preserve lines_( count_-1 )
			End If
			Property = Join$( lines_(), $CrLf )
		End Property
		Property Set Text( value As String )
			count_ = StrSplit( value, lines_(), $CrLf )
		End Property

		Method AddText( value As String )
			Local lns() As String
			Local i, c As Long

			c = StrSplit( value, lns(), $CrLf )

			ReDim Preserve lines_( count_ + c - 1 )
			For i = 0 To c-1
				lines_( count_ + i ) = lns( i )
			Next
			count_ += c
		End Method

		Method InsertText( ByVal lineNumber As Long, value As String )
			Local lns() As String
			Local i, c As Long

			If lineNumber<0 Then Exit Method
			If lineNumber>count_ Then Exit Method
			c = StrSplit( value, lns(), $CrLf )

			count_ += c
			ReDim Preserve lines_( count_-1 )
			' Make room for new lines
			For i = count_ - 1 To lineNumber + c Step -1
				Swap lines_( i ), lines_( i-c )
			Next
			' Fill in new lines
			For i=0 To c - 1
				lines_( lineNumber+i ) = lns( i )
			Next
		End Method

		Method DeleteText( ByVal lineNumber As Long, ByVal cnt As Long )
			Local i As Long

			If lineNumber<0 Then Exit Method
			If lineNumber>=count_ Then Exit Method
			If lineNumber+cnt >= count_ Then cnt = count_ - lineNumber

			For i=lineNumber To count_-cnt-1
				Swap lines_(i), lines_(i+cnt)
			Next
			For i = count_-cnt To count_-1
				lines_(i) = ""
			Next
			count_ -= cnt
		End Method

		Method ReplaceText( ByVal lineNumber As Long, ByVal cnt As Long, value As String )
			Local lns() As String
			Local i, c, addLines, delLines As Long

			If lineNumber<0 Or lineNumber>=count_ Then Exit Method
			If lineNumber+cnt >= count_ Then cnt = count_ - lineNumber

			c = StrSplit( value, lns(), $CrLf )

			Local v As String
			v = Join$( lines_(), $CrLf )

			If c>cnt Then
				' Adding lines
				addLines = c-cnt
				ReDim Preserve lines_( count_ + addLines - 1 )
				' Make room for lines
				For i = count_ + addLines - 1 To lineNumber + c Step -1
					lines_( i ) = lines_( i-addLines )
					v = Join$( lines_(), $CrLf )
				Next
				' Fill in lines
				For i=0 To c-1
					lines_( i+lineNumber ) = lns(i)
					v = Join$( lines_(), $CrLf )
				Next
				v = Join$( lines_(), $CrLf )
				count_ += addLines
			Else
				' Removing some lines
				delLines = cnt-c
				' Fill in what we have
				For i=0 To c-1
					lines_( lineNumber+i ) = lns(i)
				Next
				lineNumber += c
				' remove extra lines
				For i=lineNumber To count_-delLines-1
					lines_( i ) = lines_( i+delLines )
				Next
				count_ -= delLines
			End If
		End Method

		Method InsertChars( ByVal lineNumber As Long, ByVal index As Long, insertString As String )
			If lineNumber<0 Or lineNumber>=count_ Then Exit Method
			If index<0 Or index>Len( lines_( lineNumber ) ) Then Exit Method
			lines_( lineNumber ) = Left$( lines_( lineNumber ), index ) + insertString + Mid$( lines_( lineNumber ), index+1 )
		End Method

		Method DeleteChars( ByVal lineNumber As Long, ByVal index As Long, ByVal cnt As Long )
			If lineNumber<0 Or lineNumber>=count_ Then Exit Method
			If index<0 Or index >= Len( lines_( lineNumber ) ) Then Exit Method

			If index=0 Then
				lines_( lineNumber ) = Clip$( Left, lines_( lineNumber ), cnt )
			Else
				lines_( lineNumber ) = Clip$( Mid, lines_( lineNumber ), index+1, cnt )
			End If
		End Method

		Method ReplaceChars( ByVal lineNumber As Long, ByVal index As Long, ByVal cnt As Long, insertString As String )
			If lineNumber<0 Or lineNumber>=count_ Then Exit Method
			If index<0 Or index>Len( lines_(lineNumber) ) Then Exit Method
			lines_( lineNumber ) = Left$( lines_( lineNumber ), index ) + insertString + Mid$( lines_( lineNumber ), index+cnt+1 )
		End Method

		Property Get LinePointer( ByVal lineNumber As Long ) As Dword
			If lineNumber < 0 Or lineNumber >= count_ Then Exit Property
			Property = StrPtr( lines_( lineNumber ) )
		End Property

		Property Get Line( ByVal lineNumber As Long ) As String
			If lineNumber < 0 Or lineNumber >= count_ Then Exit Property
			Property = lines_( lineNumber )
		End Property
		Property Set Line( ByVal lineNumber As Long, value As String )
			If lineNumber < 0 Or lineNumber >= count_ Then Exit Property
			lines_(lineNumber) = value
		End Property

		Property Get Filename() As String
			Property = filename_
		End Property
		Property Set Filename( value As String )
			filename_ = value
		End Property

		Method Reload()
			Local v As String

			If IsFalse IsFile( filename_ ) Then Exit Method

			Open filename_ For Binary As #1
			Get$ #1, Lof(1), v
			Close #1

			count_ = StrSplit( v, lines_(), $CrLf )
		End Method

		Method Save( Optional filename As String )
			Local saveFilename As String
			Local f As Long

			If isExternal_ Then Exit Method: ' Don't save external files!

			If IsMissing( filename ) Then
				saveFilename = filename_
			Else
				saveFilename = filename
			End If

			Call EnsureFileAndPath( saveFilename )

			If count_ <=0 Then
				f = FreeFile
				Open saveFilename For Output As #f
				Close #f
				Exit Method: ' Empty file
			End If

			If UBound( lines_() ) <> count_-1 Then
				ReDim Preserve lines_( count_-1 )
			End If

			Open saveFilename For Output As #1
			Print #1, Join$( lines_(), $CrLf );
			Close
		End Method
	End Interface
End Class
