'===================================================================================
' This program runs tests against the PbSlice source code to help ensure it's
' functioning correctly.  When a problem has been found, add a new test for it,
' fix the problem and re-run the tests.
'
' This is also an example of how to include the PbSlice source code in your project.
' Just ensure step 1 is done once with each compiling environment you use, ensure
' step 2 has been done once on the computer.
'
' Note: While this program works with PBWin, it works best with PBCC which allows
'       scrolling of the text.  Also PBCC has a much faster display.
'
'-----------------------------------------------------------------------------------
' To compile ensure:
'-----------------------------------------------------------------------------------
' 1) The IDE/Compiler search path includes the full path to the src sub directory
' 2) Ensure src\PbSlice.inc has the path you placed the source in.  Replace ALL
'    c:\Prj\Pb\PbSlice2\Src with the directory you're copy of the source is.
' 3) Ensure test\TestProject\test.bas has the appropriate paths. ie. change
'    c:\prj\pb\PbSlice2\test\TestProject to the directory where you have
'    your test files.
'
' For usage information pbsTest /?
'
'-----------------------------------------------------------------------------------
' Testing notes:
'-----------------------------------------------------------------------------------
' When running generally, set %DEBUG=1 in PbSlice.inc.
'
' If you want to run a trace, set %DEBUG=0 in PbSlice.inc and uncomment Trace On
' below.
'
' If you want to run a profile, you'll probabbly want to add debugging aliases
' and then uncomment Profile near the bottom.  Easiest way to add debugging
' aliases is to update pbsFix for debugging aliases and run it.  See the comments
' in the code.
'
' If you want to follow class creation/destruction in PbSlice.inc set
' %LOG_CREATE=1.
'
' If you want to see all the debug messages, in PbSlice.inc uncomment '%REVIEW=1
'
' If you don't want to use the API routines, in PbSlice.inc set %USE_API=0
'
' To leave the temporary directories and files created by a debugging run, set
' %CLEAN_AFTER=0 below.
'===================================================================================

#Compile Exe
#Compiler PBWin 10, PBCC 6
#Dim All
#Debug Display On
#Debug Error On
#Optimize Code Off
#Tools On

' Set to where you put this test project
$BASE_PATH = "C:\prj\pb\PBSlice2\test\"

' File used for declare test
$DECLARE_FILE = $BASE_PATH + "TestProject\testindent.bas"
'$DECLARE_FILE = $BASE_PATH + "PbSliceTest.bas"

' File used for HTML/BBCode/Standardize test
$FORMAT_FILE = $BASE_PATH + "TestProject\testformat.bas"

' 0=Leave directorys, 1=Leave only TestProject
%CLEAN_AFTER=1

' Test harness
Global basePath As String
Global exitMessage As String

' Using PbSlice source for tests (Allows debugging)
#Include "PBSlice2.inc"

' Test support
#Include Once "src\support.inc"

' Interactive test
#Include Once "src\declareTest.inc"

' Visual tests
#Include Once "src\aliasTest.inc"
#Include Once "src\bbcodeTest.inc"
#Include Once "src\casingTest.inc"
#Include Once "src\exportsTest.inc"
#Include Once "src\genIncTest.inc"
#Include Once "src\htmlTest.inc"
#Include Once "src\interfaceTest.inc"

' Unit tests
#Include Once "src\unitFileTest.inc"
#Include Once "src\unitFilesystemTest.inc"
#Include Once "src\unitKeywordsTest.inc"
#Include Once "src\unitProjectTest.inc"
#Include Once "src\unitSnippetTest.inc"
#Include Once "src\unitStringTest.inc"
#Include Once "src\unitTokensTest.inc"

#If %Def( %PB_Win32 )
Macro NEWWIN
	MacroTemp hWin
	Local hWin As Long
	Txt.Window("PbSlice Test", 10, 10, 50,80) To hWin
End Macro
Macro PRNT(a)
	Txt.Print a
End Macro
#Else
Macro NewWIN
End Macro
Macro PRNT(a)
	Con.Print a
End Macro
#EndIf

Function PBMain () As Long
	Local value As String
	Local codeText As String
	Local i As Long
	Local a, b As String
	Local started, ended As Single

	NEWWIN

	Call DoCleanup()

	Trace New "pbsTest_Trace.txt"
	'Trace On

	For i = 1 To 100
		value = Command$( i )
		If value = "" Then Exit For
		If Left$( value, 1 ) = "-" Then value = "/"+Mid$( value,2)


		started = Timer
		Select Case LCase$( Left$( value, 4) )
			Case "/cas":  Call CasingTest()
			Case "/bbc":  Call BBCodeTest()
			Case "/htm":  Call HtmlTest()
			Case "/dec":  Call DeclareTests()
			Case "/exp":  Call ExportsTest()
			Case "/inc":  Call GenIncTest()
			Case "/int":  Call InterfaceTest()
			Case "/ali":  Call AliasTest()
			Case "/?", "/h","/help"
				PRNT("Usage: pbsTest [/ali] [/bbc] [/cas] [/dec] [/exp] [/htm] [/inc] [/int] [/fil] [/fsy] [/key] [/pro] [/sni] [/str] [/tok] [/all] [/?] [/h] [/hel]" )
				PRNT("")
				PRNT("/ali - Add Debug/export aliases, test alias removal")
				PRNT("/bbc - Test formatting as BBCode")
				PRNT("/cas - Test casing text")
				PRNT("/dec - Interactive review of declarations")
				PRNT("/exp - View exports")
				PRNT("/htm - Test formatting as html")
				PRNT("/inc - Generates SLL and DLL include files")
				PRNT("/int - Small interface test")

				PRNT("/fil - File unit tests")
				PRNT("/fsy - Filesystem unit tests")
				PRNT("/key - Keyword unit tests")
				PRNT("/pro - Project unit tests")
				PRNT("/sni - Snippet unit tests")
				PRNT("/str - String unit tests")
				PRNT("/tok - Token unit tests")

				PRNT("/all - Run all unit tests")

				PRNT("/?   - Display this help")
				PRNT("/h   - Display this help")
				PRNT("/hel - Display this help")
				Exit For
			' Non-visual
			Case "/pro":  Call ProjectTests()
			Case "/fil":  Call FileTests()

			Case "/fil":  Call FileTests()
			Case "/fsy":  Call FilesystemTests()
			Case "/key":  Call KeywordTests()
			Case "/pro":  Call ProjectTests()
			Case "/sni":  Call SnippetTests()
			Case "/str":  Call Test_String()
			Case "/tok":  Call Token_Tests()

			Case "/all":  i = 1: Exit For
		End Select
	Next
	If i=1 Then
		Call FileTests()
		Call FilesystemTests()
		Call KeywordTests()
		Call ProjectTests()
		Call SnippetTests()
		Call Test_String()
		Call Token_Tests()
	End If
	ended = Timer
	Debug_Log("Elapsed: " + Format$(ended-started, "#,##0.000") + " seconds")
	If %CLEAN_AFTER Then Call DoCleanup()

	If exitMessage="exit" Then Exit Function: ' For interactive tests

	'Profile "pbsTest_Profile.txt"

	PrintList( exitMessage+"Press ENTER to exit")
	Trace Off
	Trace Close
#If %Def( %PB_CC32 )
	Con.Line.Input a
#ElseIf %Def( %PB_Win32 )
	Txt.Line.Input a
#EndIf
End Function
